/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 *  Marc Ordinas i Llopis, Collabora Ltd <marc.ordinasillopis@linaro.org>
 */

#include <X11/Xlib.h>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <getopt.h>

#include "options.h"

int Options::size = 512;
std::list<Window> Options::window_ids;
bool Options::run_forever = false;
bool Options::use_accel_tfp = true;
std::list<std::string> Options::benchmarks;
bool Options::draw = true;
bool Options::idle_redraw = false;
bool Options::force_tex_update = false;
bool Options::manual_redirect = false;
bool Options::show_debug = false;
bool Options::show_help = false;
bool Options::list_tests = false;
bool Options::swap_buffers = true;
#ifdef USE_GLPROXY
Options::Backends Options::backend = BACKEND_EGL_ES2;
#elif defined(USE_GLX)
Options::Backends Options::backend = BACKEND_GLX;
#elif defined(USE_EGL) && defined(USE_GL)
Options::Backends Options::backend = BACKEND_EGL_GL;
#elif defined(USE_EGL) && defined(USE_GLES2)
Options::Backends Options::backend = BACKEND_EGL_ES2;
#endif

static struct option long_options[] = {
    {"ids", 1, 0, 0},
    {"size", 1, 0, 0},
    {"no-accel-tfp", 0, 0, 0},
    {"benchmark", 1, 0, 0},
    {"no-draw", 0, 0, 0},
    {"idle-redraw", 0, 0, 0},
    {"force-tex-update", 0, 0, 0},
    {"manual-redirect", 0, 0, 0},
    {"run-forever", 0, 0, 0},
    {"list-tests", 0, 0, 0},
    {"no-swap-buffers", 0, 0, 0},
    {"debug", 0, 0, 0},
    {"help", 0, 0, 0},
    {"backend", 1, 0, 0},
    {0, 0, 0, 0}
};

#ifdef USE_GLPROXY
static const char *available_backends = "glx, egl-es2, egl-gl";
#elif defined(USE_GLX)
static const char *available_backends = "glx";
#elif defined(USE_EGL) && defined(USE_GL)
static const char *available_backends = "egl-gl";
#elif defined(USE_EGL) && defined(USE_GLES2)
static const char *available_backends = "egl-es2";
#else
static const char *available_backends = "";
#endif

static void
parse_window_ids(const char *str, std::list<Window> &list)
{
    const char *cur = str;

    /* Split the id string and redirect each window */
    while (cur != NULL && *cur != '\0') {
        Window id = strtoul(cur, (char **)&cur, 0);
        if (*cur != '\0')
            ++cur;
        list.push_back(id);
    }
}

static Options::Backends
parse_backend(const char *str)
{
    if (!strstr(available_backends, str))
        return Options::BACKEND_NONE;

    if (!strcmp("glx", str))
        return Options::BACKEND_GLX;
    else if (!strcmp("egl-gl", str))
        return Options::BACKEND_EGL_GL;
    else if (!strcmp("egl-es2", str))
        return Options::BACKEND_EGL_ES2;

    return Options::BACKEND_NONE;
}

void
Options::print_help()
{
    printf("An OpenGL (ES) 2.0 compositing benchmark\n"
           "\n"
           "Options:\n"
           "  -i, --ids IDs          A comma separated list of window ids to track\n"
           "                         (base 10 or 16)\n"
           "  -s, --size SIZE        The size of each side of the main window in pixels\n"
           "                         [default: 512]\n"
           "      --no-accel-tfp     Don't use accelerated TFP (use glTexImage2D)\n"
           "  -b, --benchmark BENCH  A benchmark to run: 'test(:opt1=val1)*'\n"
           "                         (the option can be used multiple times)\n"
           "      --no-draw          Process the windows but don't draw anything on screen\n"
           "      --idle-redraw      Redraw when idle, even when nothing has changed\n"
           "      --force-tex-update Force texture updates even when they are unchanged\n"
           "      --manual-redirect  Perform a manual redirect when using XComposite\n"
           "      --run-forever      Run indefinitely, looping from the last test back to the first\n"
           "  -l, --list-tests       List the avalaible tests and their options\n"
           "      --no-swap-buffers  Don't update the screen by swapping the front and\n"
           "                         back buffer, use glFinish() instead\n"
           "      --backend BACKEND  The OpenGL backend to use (%s)\n"
           "  -d, --debug            Display debug messages\n"
           "  -h, --help             Display help\n",
           available_backends);
}

bool
Options::parse_args(int argc, char **argv)
{
    while (1) {
        int option_index = -1;
        int c;
        const char *optname = "";

        c = getopt_long(argc, argv, "i:s:b:ldh",
                        long_options, &option_index);
        if (c == -1)
            break;
        if (c == ':' || c == '?')
            return false;

        if (option_index != -1)
            optname = long_options[option_index].name;

        if (c == 'i' || !strcmp(optname, "ids"))
            parse_window_ids(optarg, Options::window_ids);
        else if (c == 's' || !strcmp(optname, "size"))
            Options::size = strtol(optarg, NULL, 10);
        else if (!strcmp(optname, "no-accel-tfp"))
            Options::use_accel_tfp = false;
        else if (c == 'b' || !strcmp(optname, "benchmark"))
            Options::benchmarks.push_back(optarg);
        else if (!strcmp(optname, "no-draw"))
            Options::draw = false;
        else if (!strcmp(optname, "idle-redraw"))
            Options::idle_redraw = true;
        else if (!strcmp(optname, "force-tex-update"))
            Options::force_tex_update = true;
        else if (!strcmp(optname, "run-forever"))
            Options::run_forever = true;
        else if (c == 'l' || !strcmp(optname, "list-tests"))
            Options::list_tests = true;
        else if (!strcmp(optname, "manual-redirect"))
            Options::manual_redirect = true;
        else if (!strcmp(optname, "no-swap-buffers"))
            Options::swap_buffers = false;
        else if (c == 'd' || !strcmp(optname, "debug"))
            Options::show_debug = true;
        else if (c == 'h' || !strcmp(optname, "help"))
            Options::show_help = true;
        else if (!strcmp(optname, "backend"))
            Options::backend = parse_backend(optarg);
    }

    return true;
}
