/*
 * Copyright © 2012 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */
#ifndef RENDER_OBJECT_H_
#define RENDER_OBJECT_H_

#include "vec.h"
#include "program.h"

//
// A RenderObject represents a source and target of rendering
// operations.
//
class RenderObject
{
public:
    RenderObject() :
        texture_(0), fbo_(0), rotation_rad_(0),
        texture_contents_invalid_(true) {}

    // Public interfaces with default implementations
    virtual void init(Program& default_program);
    virtual void release();
    virtual void resize(const LibMatrix::vec2& size);
    virtual void clear();
    virtual void render_to(RenderObject& target);
    virtual void render_to(RenderObject& target, Program& program);

    // Public methods
    void make_current();

    // Get/set methods
    void position(const LibMatrix::vec2& pos) { pos_ = pos; }
    const LibMatrix::vec2& position() { return pos_; }
    const LibMatrix::vec2& size() { return size_; }
    unsigned int texture() { return texture_; }

    // Normalizes a position from [0, size] to [-1.0, 1.0]
    LibMatrix::vec2 normalize_position(const LibMatrix::vec2& pos)
    {
        return pos * 2.0 / size_ - 1.0;
    }

    // Normalizes a position from [0, size] to [0.0, 1.0]
    LibMatrix::vec2 normalize_texcoord(const LibMatrix::vec2& pos)
    {
        return pos / size_;
    }

    void rotation(float degrees)
    {
        rotation_rad_ = (M_PI * degrees / 180.0);
    }

protected:
    static bool load_shaders_from_strings(Program& program,
        const std::string& vtx_shader, const std::string& frg_shader);
    static void draw_quad_with_program(const float *position, const float *texcoord,
                                Program &program);

    LibMatrix::vec2 pos_;
    LibMatrix::vec2 size_;
    unsigned int texture_;
    unsigned int fbo_;
    Program* default_program_;

private:
    float rotate_x(float x, float y)
    {
        return x * cos(rotation_rad_) - y * sin(rotation_rad_);
    }

    float rotate_y(float x, float y)
    {
        return x * sin(rotation_rad_) + y * cos(rotation_rad_);
    }

    float rotation_rad_;
    bool texture_contents_invalid_;
};

//
// A RenderObject representing the screen.
//
// Rendering to this objects renders to the screen framebuffer.
//
class RenderScreen : public RenderObject
{
public:
    // Our own implementation of the init and release interfaces.
    virtual void init(Program& program) { default_program_ = &program; }
    virtual void release() {}
};

//
// A RenderObject with a background image.
//
// The image is drawn to the RenderObject automatically when the
// object is cleared, resized etc. The background texture for this object
// can either be an image loaded from a named file (in which case the texture
// name is created during the load), or it can be set from an existing texture
// object.
//
class RenderClearImage : public RenderObject
{
public:
    RenderClearImage();
    RenderClearImage(const std::string& image_file);

    // Our own implementation of the init interface.
    virtual void init(Program& program);

    // Sets the background texture name.
    void set_background(unsigned int background_texture);

    // "Clears" the object to the contents of the background texture. 
    void clear();

private:
    std::string background_texture_name_;
    unsigned int background_texture_;
    bool background_is_image_;
    // Vertex position is always the same for any rendering with this object,
    // however, texture coordinates vary depending upon whether the texture
    // came from a static image loaded from a file (for example) or a window
    // system pixmap (inverted sense of y direction).
    const float* texcoord_;
    static const float position_[8];
    static const float texcoord_image_[8];
    static const float texcoord_window_[8];
};

#endif // RENDER_OBJECT_H_
